@@ -3,7 +3,7 @@
 /*
  *  File-Roller
  *
- *  Copyright (C) 2001, 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2001, 2003, 2004 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include "file-utils.h"
+#include "fr-stock.h"
 #include "window.h"
 #include "gtk-utils.h"
 
@@ -39,12 +40,11 @@ typedef struct {
 	GtkWidget *include_subfold_checkbutton;
 	GtkWidget *add_if_newer_checkbutton;
 	GtkWidget *exclude_symlinks;
-	GtkWidget *exclude_other_fs;
-	GtkWidget *exclude_backup_files;
-	GtkWidget *exclude_dot_files;
-	GtkWidget *exclude_files_checkbutton;
+	GtkWidget *include_files_checkbutton;
+	GtkWidget *include_files_entry;
+	GtkWidget *include_files_label;
 	GtkWidget *exclude_files_entry;
-	GtkWidget *ignore_case;
+	GtkWidget *exclude_files_label;
 	GtkWidget *load_button;
 	GtkWidget *save_button;
 } DialogData;
@@ -55,7 +55,6 @@ open_file_destroy_cb (GtkWidget *w,
 		      GtkWidget *file_sel)
 {
 	DialogData *data;
-
 	data = g_object_get_data (G_OBJECT (file_sel), "fr_dialog_data");
 	g_free (data);
 }
@@ -80,36 +79,37 @@ utf8_only_spaces (const char *text)
 
 
 static int
-open_file_ok_cb (GtkWidget *w,
-                 GtkWidget *file_sel)
+file_sel_response_cb (GtkWidget      *w,
+		      int             response,
+		      GtkFileChooser *file_sel)
 {
 	FRWindow   *window;
-	const char *file_sel_path;
-	char       *path;
+	char       *current_folder;
+	gboolean    update, recursive, follow_links;
+	GSList     *selections, *iter;
+	GList      *item_list = NULL;
 	DialogData *data;
-	gboolean    update;
-	char       *base_dir;
 
 	data = g_object_get_data (G_OBJECT (file_sel), "fr_dialog_data");
+
+	if (response == GTK_RESPONSE_CANCEL) {
+		gtk_widget_destroy (data->dialog);
+		return TRUE;
+	}
+
 	window = data->window;
 
-	file_sel_path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel));
+	current_folder = gtk_file_chooser_get_current_folder (file_sel);
 
-        if (file_sel_path == NULL)
-                return FALSE;
+	/* check folder permissions. */
 
-	path = remove_ending_separator (file_sel_path);
-	base_dir = remove_level_from_path (path);
-
-	/* check directory permissions. */
-
-	if (path_is_dir (base_dir) 
-	    && access (base_dir, R_OK | X_OK) != 0) {
+	if (path_is_dir (current_folder) 
+	    && access (current_folder, R_OK | X_OK) != 0) {
 		GtkWidget *d;
 		char      *utf8_path;
 		char      *message;
 
-		utf8_path = g_filename_to_utf8 (base_dir, -1, NULL, NULL, NULL);
+		utf8_path = g_filename_to_utf8 (current_folder, -1, NULL, NULL, NULL);
 		message = g_strdup_printf (_("You don't have the right permissions to read files from folder \"%s\""), utf8_path);
 		g_free (utf8_path);
 
@@ -125,136 +125,111 @@ open_file_ok_cb (GtkWidget *w,
 		gtk_widget_destroy (GTK_WIDGET (d));
 		g_free (utf8_path);
 
-		g_free (base_dir);
-		g_free (path);
+		g_free (current_folder);
 
 		return FALSE;
 	}
 
-	window_set_add_default_dir (window, base_dir);
+	window_set_add_default_dir (window, current_folder);
 
 	update = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->add_if_newer_checkbutton));
+	recursive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->include_subfold_checkbutton));
+	follow_links = ! gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_symlinks));
 
-	if ((strchr (path, '*') == NULL) && (strchr (path, '?') == NULL)) {
-		GList  *files;
-		char  **scan;
-		char  **selections;
+	/**/
 
-		selections = gtk_file_selection_get_selections (GTK_FILE_SELECTION (file_sel));
+	selections = gtk_file_chooser_get_filenames (file_sel);
+	for (iter = selections; iter != NULL; iter = iter->next) {
+		char *path = iter->data;
+		item_list = g_list_prepend (item_list, path);
+	}
+	
+	if ((item_list == NULL)) {
+		const char *include_files = gtk_entry_get_text (GTK_ENTRY (data->include_files_entry));
+		const char *exclude_files = gtk_entry_get_text (GTK_ENTRY (data->exclude_files_entry));
 
-		files = NULL;
-		if (*selections == NULL)
-                        files = g_list_prepend (files, (gpointer) file_name_from_path (path));
+		if (utf8_only_spaces (include_files) 
+		    && utf8_only_spaces (exclude_files))
+			return FALSE;
 
-		for (scan = selections; *scan != NULL; scan++) {
-			files = g_list_prepend (files, (gpointer) file_name_from_path (*scan));
-#ifdef DEBUG
-			g_print ("add %s\n", *scan);
-#endif
-		}
-
-#ifdef DEBUG
-		g_print ("basedir : %s\n", base_dir);
-#endif
-
-		if (files != NULL) {
-			char *first = files->data;
-			char *first_path;
-
-			first_path = g_build_path (G_DIR_SEPARATOR_S, base_dir, first, NULL);
-			if (path_is_dir (first_path)) {
-				window_archive_add_directory (window,
-							      file_name_from_path (first), 
-							      base_dir, 
-							      FALSE,
-							      window->password,
-							      window->compression);
-
-			} else 
-				window_archive_add (window,
-						    files,
-						    base_dir,
-						    window_get_current_location (window),
-						    update,
-						    window->password,
-						    window->compression);
-
-			g_free (first_path);
-		}
-		
-		g_list_free (files);
-		g_strfreev (selections); 
-
-	} else {
-		gboolean    recursive;
-		gboolean    no_symlinks;
-		gboolean    same_fs;
-		gboolean    no_backup_files;
-		gboolean    no_dot_files;
-		gboolean    ignore_case;
-		char       *include_files;
-		const char *exclude_files;
-
-		recursive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->include_subfold_checkbutton));
-		no_symlinks = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_symlinks));
-		same_fs = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_other_fs));
-		no_backup_files = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_backup_files));
-		no_dot_files = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_dot_files));
-		ignore_case = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ignore_case));
-
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_files_checkbutton))) {
-			exclude_files = gtk_entry_get_text (GTK_ENTRY (data->exclude_files_entry));
-			if (utf8_only_spaces (exclude_files)) 
-				exclude_files = NULL;
-		} else
+		if (utf8_only_spaces (include_files))
+			include_files = "*";
+		if (utf8_only_spaces (exclude_files))
 			exclude_files = NULL;
 
-		include_files = g_filename_to_utf8 (file_name_from_path (path), 
-						  -1, 0, 0, 0);
 		window_archive_add_with_wildcard (window,
 						  include_files,
 						  exclude_files,
-						  base_dir,
+						  current_folder,
 						  update,
 						  recursive,
-						  ! no_symlinks,
-						  same_fs,
-						  no_backup_files,
-						  no_dot_files,
-						  ignore_case,
+						  follow_links,
 						  window->password,
 						  window->compression);
-		g_free (include_files);
-	}
 
-	g_free (path);
-	g_free (base_dir);
-	gtk_widget_destroy (file_sel);
+	} else 
+		window_archive_add_items (window, item_list, update);
+
+	g_list_free (item_list);
+	g_slist_foreach (selections, (GFunc) g_free, NULL);
+	g_slist_free (selections);
+	g_free (current_folder);
+
+	gtk_widget_destroy (data->dialog);
 
 	return TRUE;
 }
 
 
 static void
-selection_entry_changed (GtkWidget  *widget, 
-			 DialogData *data)
+update_sensitivity (DialogData *data)
 {
-	const char *path;
-	gboolean    wildcard;
+	GSList   *selections;
+	gboolean  can_add_wildcard, wildcard_specified = FALSE;
 
-	path = gtk_entry_get_text (GTK_ENTRY (GTK_FILE_SELECTION (data->dialog)->selection_entry));
-	wildcard = ((g_utf8_strchr (path, -1, '*') != NULL) 
-		    || (g_utf8_strchr (path, -1, '?') != NULL));
+	selections = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (data->dialog));
+	can_add_wildcard = (selections == NULL);
 
-	gtk_widget_set_sensitive (data->include_subfold_checkbutton, wildcard);
-	gtk_widget_set_sensitive (data->exclude_symlinks, GTK_TOGGLE_BUTTON (data->include_subfold_checkbutton)->active && wildcard);
-	gtk_widget_set_sensitive (data->exclude_other_fs, GTK_TOGGLE_BUTTON (data->include_subfold_checkbutton)->active && wildcard);
-	gtk_widget_set_sensitive (data->exclude_backup_files, wildcard);
-	gtk_widget_set_sensitive (data->exclude_dot_files, wildcard);
-	gtk_widget_set_sensitive (data->exclude_files_checkbutton, wildcard);
-	gtk_widget_set_sensitive (data->exclude_files_entry,  GTK_TOGGLE_BUTTON (data->exclude_files_checkbutton)->active && wildcard);
-	gtk_widget_set_sensitive (data->include_subfold_checkbutton, wildcard);
-	gtk_widget_set_sensitive (data->ignore_case, wildcard);
+	if (can_add_wildcard) {
+		const char *files;
+		gboolean    include_wildcard;
+		gboolean    exclude_wildcard;
+
+		files = gtk_entry_get_text (GTK_ENTRY (data->include_files_entry));
+		include_wildcard = ((g_utf8_strchr (files, -1, '*') != NULL) 
+				    || (g_utf8_strchr (files, -1, '?') != NULL));
+
+		files = gtk_entry_get_text (GTK_ENTRY (data->exclude_files_entry));
+		exclude_wildcard = ((g_utf8_strchr (files, -1, '*') != NULL) 
+				    || (g_utf8_strchr (files, -1, '?') != NULL));
+		wildcard_specified = include_wildcard || exclude_wildcard;
+	}
+
+	gtk_widget_set_sensitive (data->include_files_entry, can_add_wildcard);
+	gtk_widget_set_sensitive (data->exclude_files_entry, can_add_wildcard);
+	gtk_widget_set_sensitive (data->include_files_label, can_add_wildcard);
+	gtk_widget_set_sensitive (data->exclude_files_label, can_add_wildcard);
+	gtk_widget_set_sensitive (data->include_subfold_checkbutton, can_add_wildcard && wildcard_specified);
+	gtk_widget_set_sensitive (data->exclude_symlinks, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->include_subfold_checkbutton)) && can_add_wildcard && wildcard_specified);
+
+	g_slist_foreach (selections, (GFunc) g_free, FALSE);
+	g_slist_free (selections);
+}
+
+
+static void
+selection_changed_cb (GtkWidget  *file_sel, 
+		      DialogData *data)
+{
+	update_sensitivity (data);
+}
+
+
+static void
+wildcard_entry_changed_cb (GtkWidget  *file_sel, 
+			   DialogData *data)
+{
+	update_sensitivity (data);
 }
 
 
@@ -263,76 +238,16 @@ include_subfold_toggled_cb (GtkWidget *widget,
 			    gpointer   callback_data)
 {
 	DialogData *data = callback_data;
-
 	gtk_widget_set_sensitive (data->exclude_symlinks,
 				  GTK_TOGGLE_BUTTON (widget)->active);
-	gtk_widget_set_sensitive (data->exclude_other_fs,
-				  GTK_TOGGLE_BUTTON (widget)->active);
 	return FALSE;
 }
 
 
-static int
-exclude_files_toggled_cb (GtkWidget *widget, 
-			  gpointer   callback_data)
-{
-	DialogData *data = callback_data;
-	
-	gtk_widget_set_sensitive (data->exclude_files_entry,
-				  GTK_TOGGLE_BUTTON (widget)->active);
-	return FALSE;   
-}
-
-
 static void load_options_cb (GtkWidget *w, GtkWidget *data);
 static void save_options_cb (GtkWidget *w, GtkWidget *data);
 
 
-static GtkWidget *
-create_button (const char *stock_id, 
-               const char *text)
-{
-        GtkWidget    *button;
-        GtkWidget    *hbox;
-        GtkWidget    *image;
-        GtkWidget    *label;
-        GtkWidget    *align;
-	const char   *label_text;
-        gboolean      text_is_stock;
-        GtkStockItem  stock_item;
-
-        button = gtk_button_new ();
-
-        if (gtk_stock_lookup (text, &stock_item)) {
-                label_text = stock_item.label;
-                text_is_stock = TRUE;
-        } else {
-		label_text = text;
-		text_is_stock = FALSE;
-        }
-
-        if (text_is_stock)
-                image = gtk_image_new_from_stock (text, GTK_ICON_SIZE_BUTTON);
-        else
-                image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
-        label = gtk_label_new_with_mnemonic (label_text);
-        hbox = gtk_hbox_new (FALSE, 2);
-        align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-
-	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
-        gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
-
-        gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-        gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-        gtk_container_add (GTK_CONTAINER (button), align);
-        gtk_container_add (GTK_CONTAINER (align), hbox);
-
-        gtk_widget_show_all (button);
-
-        return button;
-}
-
-
 /* create the "add" dialog. */
 void
 add_cb (GtkWidget *widget, 
@@ -343,84 +258,93 @@ add_cb (GtkWidget *widget,
 	gchar       *dir;
 	GtkWidget   *main_box;
 	GtkWidget   *vbox, *hbox;
-	GtkWidget   *frame;
+	GtkWidget   *table;	
 	GtkTooltips *tooltips;
 	
 	tooltips = gtk_tooltips_new ();
  
 	data = g_new (DialogData, 1);
 	data->window = callback_data;
-	data->dialog = file_sel = gtk_file_selection_new (_("Add"));
-	gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (file_sel));
-
-	gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (file_sel), TRUE);
+	data->dialog = file_sel = 
+		gtk_file_chooser_dialog_new (_("Add"),
+					     GTK_WINDOW (data->window->app),
+					     GTK_FILE_CHOOSER_ACTION_OPEN,
+					     GTK_STOCK_CANCEL, 
+					     GTK_RESPONSE_CANCEL,
+					     FR_STOCK_ADD, 
+					     GTK_RESPONSE_OK,
+					     NULL);
+	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (file_sel), TRUE);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), TRUE);
 
 	data->add_if_newer_checkbutton = gtk_check_button_new_with_mnemonic (_("_Add only if newer"));
 	data->include_subfold_checkbutton = gtk_check_button_new_with_mnemonic (_("_Include subfolders"));
 	data->exclude_symlinks = gtk_check_button_new_with_mnemonic (_("Exclude folders that are symbolic lin_ks"));
-	data->exclude_other_fs = gtk_check_button_new_with_mnemonic (_("Exclude o_ther file systems"));
-	data->exclude_backup_files = gtk_check_button_new_with_mnemonic (_("Exclude _backup files (*~)"));
-	data->exclude_dot_files = gtk_check_button_new_with_mnemonic (_("Exclude _hidden files (.*)"));
-	data->exclude_files_checkbutton = gtk_check_button_new_with_mnemonic (_("E_xclude files:"));
+
+	data->include_files_entry = gtk_entry_new ();
+	gtk_tooltips_set_tip (tooltips, data->include_files_entry, _("example: *.o; *.bak"), NULL);
+	data->include_files_label = gtk_label_new_with_mnemonic (_("_Include files:"));
+	gtk_misc_set_alignment (GTK_MISC (data->include_files_label), 0.0, 0.5);
+
 	data->exclude_files_entry = gtk_entry_new ();
 	gtk_tooltips_set_tip (tooltips, data->exclude_files_entry, _("example: *.o; *.bak"), NULL);
-	data->ignore_case = gtk_check_button_new_with_mnemonic (_("Igno_re case"));
-	data->load_button = create_button (GTK_STOCK_OPEN, _("_Load Options"));
-        data->save_button = create_button (GTK_STOCK_SAVE, _("Sa_ve Options"));
+	data->exclude_files_label = gtk_label_new_with_mnemonic (_("E_xclude files:"));
+	gtk_misc_set_alignment (GTK_MISC (data->exclude_files_label), 0.0, 0.5);
 
-	frame = GTK_FILE_SELECTION (file_sel)->action_area;
+	data->load_button = gtk_button_new_with_mnemonic (_("_Load Options"));
+	data->save_button = gtk_button_new_with_mnemonic (_("Sa_ve Options"));
 
 	main_box = gtk_hbox_new (FALSE, 5);
 	gtk_container_set_border_width (GTK_CONTAINER (main_box), 0);
-	gtk_container_add (GTK_CONTAINER (frame), main_box);
+	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (file_sel), main_box);
 
-	vbox = gtk_vbox_new (FALSE, 1);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
 	gtk_box_pack_start (GTK_BOX (main_box), vbox, TRUE, TRUE, 0);
 
 	gtk_box_pack_start (GTK_BOX (vbox), data->add_if_newer_checkbutton, 
 			    TRUE, TRUE, 0);
 
+	table = gtk_table_new (2, 2, FALSE);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+	gtk_box_pack_start (GTK_BOX (vbox), table,
+			    TRUE, TRUE, 0);
+
+	gtk_table_attach (GTK_TABLE (table), 
+			  data->include_files_label,
+			  0, 1,
+			  0, 1,
+			  GTK_FILL, 0,
+			  0, 0);
+	gtk_table_attach (GTK_TABLE (table),
+			  data->include_files_entry, 
+			  1, 2,
+			  0, 1,
+			  0, 0,
+			  0, 0);
+	gtk_table_attach (GTK_TABLE (table), 
+			  data->exclude_files_label,
+			  0, 1,
+			  1, 2,
+			  GTK_FILL, 0,
+			  0, 0);
+	gtk_table_attach (GTK_TABLE (table),
+			  data->exclude_files_entry, 
+			  1, 2,
+			  1, 2,
+			  0, 0,
+			  0, 0);
+
 	gtk_box_pack_start (GTK_BOX (vbox), data->include_subfold_checkbutton,
 			    TRUE, TRUE, 0);
 
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), data->exclude_symlinks,
-			    TRUE, TRUE, 15);
+			    TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox,
 			    TRUE, TRUE, 0);
 
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), data->exclude_other_fs,
-			    TRUE, TRUE, 15);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox,
-			    TRUE, TRUE, 0);
-
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox,
-			    TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), data->exclude_backup_files,
-			    TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), data->exclude_dot_files,
-			    TRUE, TRUE, 0);
-
-	hbox = gtk_hbox_new (FALSE, 5);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox,
-			    TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), data->exclude_files_checkbutton,
-			    FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), data->exclude_files_entry,
-			    TRUE, TRUE, 0);
-	
-	gtk_box_pack_start (GTK_BOX (vbox), data->ignore_case,
-			    TRUE, TRUE, 0);
-
-	/**/
-	
-	gtk_box_pack_start (GTK_BOX (main_box), 
-			    gtk_vseparator_new (), 
-			    FALSE, FALSE, 0);
-	
 	/**/
 	
 	vbox = gtk_vbox_new (FALSE, 5);
@@ -431,28 +355,20 @@ add_cb (GtkWidget *widget,
 			    FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), data->save_button,
 			    FALSE, FALSE, 0);
+
+	gtk_widget_show_all (main_box);
 	
-	/**/
-
-	gtk_widget_show_all (frame);
-
 	/* set data */
 
 	dir = g_strconcat (data->window->add_default_dir, "/", "*", NULL);
-	gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_sel), dir);
+	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (file_sel), dir);
 	g_free (dir);
 
-	gtk_editable_select_region (GTK_EDITABLE (GTK_FILE_SELECTION (data->dialog)->selection_entry), 0, -1);
-
 	g_object_set_data (G_OBJECT (file_sel), "fr_dialog_data", data);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->include_subfold_checkbutton), FALSE);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->exclude_backup_files), TRUE);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->exclude_dot_files), TRUE);
 
-	gtk_widget_set_sensitive (data->exclude_symlinks, FALSE);
-	gtk_widget_set_sensitive (data->exclude_other_fs, FALSE);
-	gtk_widget_set_sensitive (data->exclude_files_entry, FALSE);
+	update_sensitivity (data);
 
 	/* signals */
 	
@@ -461,19 +377,23 @@ add_cb (GtkWidget *widget,
 			  G_CALLBACK (open_file_destroy_cb),
 			  file_sel);
 
-	g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (file_sel)->cancel_button),
-				  "clicked", 
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (file_sel));
-
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->ok_button),
-			  "clicked",
-			  G_CALLBACK (open_file_ok_cb), 
+	g_signal_connect (G_OBJECT (file_sel),
+			  "response",
+			  G_CALLBACK (file_sel_response_cb),
 			  file_sel);
 
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (file_sel)->selection_entry), 
+	g_signal_connect (G_OBJECT (file_sel), 
+			  "selection-changed",
+			  G_CALLBACK (selection_changed_cb),
+			  data);
+
+	g_signal_connect (G_OBJECT (data->include_files_entry), 
 			  "changed",
-			  G_CALLBACK (selection_entry_changed),
+			  G_CALLBACK (wildcard_entry_changed_cb),
+			  data);
+	g_signal_connect (G_OBJECT (data->exclude_files_entry), 
+			  "changed",
+			  G_CALLBACK (wildcard_entry_changed_cb),
 			  data);
 	
 	g_signal_connect (G_OBJECT (data->include_subfold_checkbutton),
@@ -481,11 +401,6 @@ add_cb (GtkWidget *widget,
 			  G_CALLBACK (include_subfold_toggled_cb),
 			  data);
 
-	g_signal_connect (G_OBJECT (data->exclude_files_checkbutton),
-			  "toggled", 
-			  G_CALLBACK (exclude_files_toggled_cb),
-			  data);
-	
 	g_signal_connect (G_OBJECT (data->load_button),
 			  "clicked",
 			  G_CALLBACK (load_options_cb), 
@@ -503,9 +418,6 @@ add_cb (GtkWidget *widget,
 }
 
 
-
-
-
 static gboolean
 config_get_bool (const char *config_file,
 		 const char *option)
@@ -537,8 +449,9 @@ config_get_string (const char *config_file,
 
 
 static void
-load_options_ok_cb (GtkWidget *w,
-		    GtkWidget *opt_sel)
+load_options_response_cb (GtkWidget *w,
+			  gint response,
+			  GtkWidget *opt_sel)
 {
 	GtkWidget  *file_sel;
 	DialogData *data;
@@ -550,16 +463,18 @@ load_options_ok_cb (GtkWidget *w,
 	gboolean    recursive;
 	gboolean    no_symlinks;
 	gboolean    same_fs;
-	gboolean    no_backup_files;
-	gboolean    no_dot_files;
-	gboolean    ignore_case;
 	char       *exclude_files;
+
+	if (response == GTK_RESPONSE_CANCEL) {
+		gtk_widget_destroy (opt_sel);
+		return;
+	}
 	
 	file_sel = g_object_get_data (G_OBJECT (opt_sel), "fr_file_sel");
 	data = g_object_get_data (G_OBJECT (file_sel), "fr_dialog_data");
 	window = data->window;
 
-	opt_file_path = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (opt_sel)));
+	opt_file_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (opt_sel));
 	
         if (opt_file_path == NULL) {
 		g_free (opt_file_path);
@@ -576,24 +491,14 @@ load_options_ok_cb (GtkWidget *w,
 	recursive = config_get_bool (opt_file_path, "recursive");
 	no_symlinks = config_get_bool (opt_file_path, "no_symlinks");
 	same_fs = config_get_bool (opt_file_path, "same_fs");
-	no_backup_files = config_get_bool (opt_file_path, "no_backup_files");
-	no_dot_files = config_get_bool (opt_file_path, "no_dot_files");
-	ignore_case = config_get_bool (opt_file_path, "ignore_case");
-	
+
 	file_sel_path = g_strconcat (base_dir, "/", path, NULL);
 
 	/* Sync widgets with options. */
 
-	gtk_file_selection_complete (GTK_FILE_SELECTION (file_sel),
-				     file_sel_path);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->add_if_newer_checkbutton), update);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->include_subfold_checkbutton), recursive);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->exclude_symlinks), no_symlinks);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->exclude_other_fs), same_fs);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->exclude_backup_files), no_backup_files);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->exclude_dot_files), no_dot_files);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->ignore_case), ignore_case);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->exclude_files_checkbutton), ! utf8_only_spaces (exclude_files));
 	gtk_entry_set_text (GTK_ENTRY (data->exclude_files_entry), exclude_files);
 	
 	/**/
@@ -621,9 +526,14 @@ load_options_cb (GtkWidget  *w,
 
 	ensure_dir_exists (options_dir, 0700);
 	
-	opt_sel = gtk_file_selection_new (_("Load Options"));
-	gtk_file_selection_set_filename (GTK_FILE_SELECTION (opt_sel), 
-					 options_dir_slash);
+	opt_sel = gtk_file_chooser_dialog_new (_("Load Options"),
+					       NULL,
+					       GTK_FILE_CHOOSER_ACTION_OPEN,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					       NULL);
+	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (opt_sel), 
+				       options_dir_slash);
 
 	g_free (options_dir);
 	g_free (options_dir_slash);
@@ -631,15 +541,9 @@ load_options_cb (GtkWidget  *w,
 	g_object_set_data (G_OBJECT (opt_sel), "fr_file_sel", file_sel);
 	
 	/* Signals */
-	
-	g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (opt_sel)->cancel_button),
-				  "clicked", 
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (opt_sel));
-	
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (opt_sel)->ok_button),
-			  "clicked",
-			  G_CALLBACK (load_options_ok_cb), 
+	g_signal_connect (G_OBJECT (opt_sel),
+			  "response",
+			  G_CALLBACK (load_options_response_cb), 
 			  opt_sel);
 	
 	gtk_window_set_modal (GTK_WINDOW (opt_sel), TRUE);
@@ -679,8 +583,9 @@ config_set_string (const char *config_file,
 
 
 static void
-save_options_ok_cb (GtkWidget *w,
-		    GtkWidget *opt_sel)
+save_options_response_cb (GtkWidget *w,
+			  gint response,
+			  GtkWidget *opt_sel)
 {
 	DialogData *data;
 	FRWindow   *window;
@@ -693,16 +598,18 @@ save_options_ok_cb (GtkWidget *w,
 	gboolean    recursive;
 	gboolean    no_symlinks;
 	gboolean    same_fs;
-	gboolean    no_backup_files;
-	gboolean    no_dot_files;
-	gboolean    ignore_case;
 	const char *exclude_files;
-	
+
+	if (response == GTK_RESPONSE_CANCEL) {
+		gtk_widget_destroy (opt_sel);
+		return;
+	}
+
 	file_sel = g_object_get_data (G_OBJECT (opt_sel), "fr_file_sel");
 	data = g_object_get_data (G_OBJECT (file_sel), "fr_dialog_data");
 	window = data->window;
 	
-	opt_file_path = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (opt_sel)));
+	opt_file_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (opt_sel));
 
         if (opt_file_path == NULL) {
 		g_free (opt_file_path);
@@ -712,7 +619,7 @@ save_options_ok_cb (GtkWidget *w,
 	
 	/* Get options. */
 	
-	file_sel_path = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_sel)));
+	file_sel_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_sel));
 	path = remove_ending_separator (file_sel_path);
 	base_dir = remove_level_from_path (path);
 	
@@ -720,16 +627,9 @@ save_options_ok_cb (GtkWidget *w,
 	
 	recursive = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->include_subfold_checkbutton));
 	no_symlinks = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_symlinks));
-	same_fs = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_other_fs));
-	no_backup_files = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_backup_files));
-	no_dot_files = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_dot_files));
-	ignore_case = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->ignore_case));
-	
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->exclude_files_checkbutton))) {
-		exclude_files = gtk_entry_get_text (GTK_ENTRY (data->exclude_files_entry));
-		if (utf8_only_spaces (exclude_files))
-			exclude_files = NULL;
-	} else
+
+	exclude_files = gtk_entry_get_text (GTK_ENTRY (data->exclude_files_entry));
+	if (utf8_only_spaces (exclude_files))
 		exclude_files = NULL;
 	
 	/* Save options. */
@@ -741,9 +641,6 @@ save_options_ok_cb (GtkWidget *w,
 	config_set_bool   (opt_file_path, "recursive", recursive);
 	config_set_bool   (opt_file_path, "no_symlinks", no_symlinks);
 	config_set_bool   (opt_file_path, "same_fs", same_fs);
-	config_set_bool   (opt_file_path, "no_backup_files", no_backup_files);
-	config_set_bool   (opt_file_path, "no_dot_files", no_dot_files);
-	config_set_bool   (opt_file_path, "ignore_case", ignore_case);
 	gnome_config_sync ();
 
 	/**/
@@ -763,32 +660,28 @@ save_options_cb (GtkWidget  *w,
 {
 	GtkWidget *opt_sel;
 	char      *options_dir;
-	char      *options_dir_slash;
 
-	options_dir = get_home_relative_dir (RC_OPTIONS_DIR);	
-	options_dir_slash = g_strconcat (options_dir, "/", NULL);
+	options_dir = get_home_relative_dir (RC_OPTIONS_DIR);
 
 	ensure_dir_exists (options_dir, 0700);
 	
-	opt_sel = gtk_file_selection_new (_("Save Options"));
-	gtk_file_selection_set_filename (GTK_FILE_SELECTION (opt_sel), 
-					 options_dir_slash);
+	opt_sel = gtk_file_chooser_dialog_new (_("Save Options"),
+					       NULL,
+					       GTK_FILE_CHOOSER_ACTION_SAVE,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+					       NULL);
+ 
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (opt_sel), options_dir);
 
 	g_free (options_dir);
-	g_free (options_dir_slash);
 	
 	g_object_set_data (G_OBJECT (opt_sel), "fr_file_sel", file_sel);
 	
 	/* Signals */
-	
-	g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (opt_sel)->cancel_button),
-				  "clicked", 
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (opt_sel));
-	
-	g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (opt_sel)->ok_button),
-			  "clicked",
-			  G_CALLBACK (save_options_ok_cb), 
+	g_signal_connect (G_OBJECT (opt_sel),
+			  "response",
+			  G_CALLBACK (save_options_response_cb), 
 			  opt_sel);
 	
 	gtk_window_set_modal (GTK_WINDOW (opt_sel), TRUE);
